/**
* @file  serial.c
* @brief HAL APIs x interfacing to the RS232 serial port 
*/
/****************** (C) COPYRIGHT 2006 STMicroelectronics **********************
* Author              MCD Application Team, STM32 only version prepared by Prague LAB
* Date First Issued   11/13/2006
* History:
* 20/03/2008 : V2.0
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/
/* Includes ------------------------------------------------------------------*/

/* 
The serial.c file provides support for the following serial communication channels:
1) serial COM through RS232
2) virtual COM through USB
Both drivers are required to be included in the building. The selection is done at runtime
(USB is chosen if the USB-mini cable is plugged).

NOTE: for the internal test applications (except test_serial), where the virtual COM through USB 
is not required, enable the NO_VIRTUAL_COM_USED define in the config.h file for removing at 
build time any reference to the virtual COM through USB drivers.
*/

/* include the serial header file */
#include "serial.h"
#include "stm32f10x_usart.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_nvic.h"

/* USB lib support */
#include "usb_lib.h"
#include "hw_config.h"

#include "usb_desc.h"

#include <stdarg.h>

/* Typedef -------------------------------------------------------------------*/
/* Define --------------------------------------------------------------------*/
#define SERIAL_USB_BUFFER_IN_SIZE       192
/* Macro ---------------------------------------------------------------------*/
/* Variables -----------------------------------------------------------------*/
/** Description: Keep the read byte over the serial port */
u8 bRByte = '\0';
/** Description: Flag to signal data over the uart */
u8 bRByteReady = 0;

u8 usb_plugged = 0; /* flag to keep track if the USB cable is plugged or not */

/* USB library support: valid for all STR71x-91x, STM32 microcontrollers*/
extern u32 count_out;
extern u32 count_in;
extern u8 buffer_out[];

u8 buffer_out_empty = 1; /* if 1 means all the bytes in buffer_out has been processed */
u8 buffer_index = 0;

/* buffer used for writing inside the hyperterminal */
u8 serial_usb_buffer_in[SERIAL_USB_BUFFER_IN_SIZE];
u8 count_char;


/* Private function prototypes -----------------------------------------------*/
void putint(unsigned int i);

/* Functions -----------------------------------------------------------------*/


/* STR7x-STR9x */
/*******************************************************************************
* Function Name  : USB_UART_putchar
* Description    : print a char 
* Input          : char
* Output         : char
* Return         : none
*******************************************************************************/
char USB_UART_putchar(char c)
{
	/* Check if the using the virtual UART through USB or the UART */
	if (usb_plugged)
	{
		/* The USB cable is plugged into the USB-Mini connector in the STRx daugther board */
		
		/* Store the data into the buffer to be sent to the virtual com */
		serial_usb_buffer_in[count_char++] = c;
		
   }
   else /* No USB cable plugged: use the UART */
   {
   	emberSerialWaitSend(APP_SERIAL_PORT);

      USART_SendData(ST_CORE_UART, c);
 	}
}/* end USB_UART_putchar() */

/*******************************************************************************
* Function Name  : putint
* Description    : asc to int conversion (to be reviewed)
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void putint(unsigned int i)
{
	unsigned char mod=i%10;
	USB_UART_putchar(0x30+mod);
}/* end putint() */

/*******************************************************************************
* Function Name  : Virtual_Com_Setup()
* Description    : Initialize the Virtual COM through USB communication channel
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Virtual_Com_Setup()
{
	/* ********************** Configure the Virtual UART through USB communication channel */
   
   /* Configure the USB interrupts */
   USB_Interrupts_Config();
   
   /* Init the USB library */
   USB_Init(); 

}/* end Virtual_Com_Setup() */

/*******************************************************************************
* Function Name  : emberSerialInit
* Description    : serial port initialization 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void emberSerialInit(void)
{
	USART_InitTypeDef USART_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
   NVIC_InitTypeDef NVIC_InitStructure;
   
   /* Configure USART Tx as alternate function push-pull */
/* BY LUDEK
   GPIO_InitStructure.GPIO_Pin = ST_CORE_UART_GPIO_TX_PIN;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
   GPIO_Init(ST_CORE_UART_GPIO, &GPIO_InitStructure);
*/
   /* Configure USART Rx as input floating */
/* BY LUDEK
   GPIO_InitStructure.GPIO_Pin = ST_CORE_UART_GPIO_RX_PIN;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
   GPIO_Init(ST_CORE_UART_GPIO, &GPIO_InitStructure);
*/
  
   /* USART configuration ------------------------------------------------------*/
   /* USART configured as follow:
        - BaudRate = 9600 baud  
        - Word Length = 8 Bits
        - 1 Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
        - USART Clock disabled
        - USART CPOL: Clock is active low
        - USART CPHA: Data is captured on the second edge 
        - USART LastBit: The clock pulse of the last data bit is not output to 
                         the SCLK pin
   */
/* BY LUDEK
   USART_InitStructure.USART_BaudRate = 9600;
   USART_InitStructure.USART_WordLength = USART_WordLength_8b;
   USART_InitStructure.USART_StopBits =  USART_StopBits_1; 
   USART_InitStructure.USART_Parity = USART_Parity_No;
   USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
   USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
   USART_InitStructure.USART_Clock = USART_Clock_Disable;
   USART_InitStructure.USART_CPOL = USART_CPOL_Low;
   USART_InitStructure.USART_CPHA = USART_CPHA_2Edge;
   USART_InitStructure.USART_LastBit = USART_LastBit_Disable;
*/
   /* Configure the USART */
/* BY LUDEK
   USART_Init(ST_CORE_UART, &USART_InitStructure);
*/
   
   /* Enable the USART Receive interrupt: this interrupt is generated when the 
      ST_CORE_UART receive data register is not empty */
/* BY LUDEK
   USART_ITConfig(ST_CORE_UART, USART_IT_RXNE, ENABLE);
*/
   
   /* Enable ST_CORE_UART */
/* BY LUDEK
   USART_Cmd(ST_CORE_UART, ENABLE);
*/
   
   /* Enable the USART1 Interrupt */
/* BY LUDEK
   NVIC_InitStructure.NVIC_IRQChannel = ST_CORE_UART_CHANNEL;
   NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1; 
   NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   NVIC_Init(&NVIC_InitStructure);
*/
   /* Setup the virtual com through USB channel */
   Virtual_Com_Setup();

} /* end emberSerialInit() */


/*******************************************************************************
* Function Name  : emberSerialPrintf (first version: to be reviewed)
* Description    : print on serial port
* Input          : u8 serial_port, char *str, ...
* Output         : None
* Return         : None
*******************************************************************************/
void emberSerialPrintf(u8 serial_port, char *str, ...)
{
  
   u8 nibbles;
   u32 arg;
   va_list ap;
   
   if (usb_plugged)
   	count_char =0;

   va_start(ap, str);
 
   while (*str) 
   {  
     if (*str == 92)    // backslash (\)
     {
       str++;
       if (*str == 'n')
         USB_UART_putchar('\n');
       else
       {
         USB_UART_putchar(92); // backslash (\)
         USB_UART_putchar(*str);
       }  
     } 
     else    
      if (*str != '%')
       USB_UART_putchar(*str);           
     else 
     { 
       u32 arg1;
       int k;
       u32 pow = 1;
       str++;  
       switch (*str)
       {        
#ifndef CODECUT			 
         case 'd':
           arg = va_arg(ap, u32);
                
           if (arg & 0x80000000)
           {
             USB_UART_putchar('-');
             arg *= -1; 
           }     
           while ((u32)(arg/(pow*10)))

             pow*=10;
           do{  
               putint(arg/pow);
               pow/=10;
             } while (pow >= 1);  
         break;
#endif // CODECUT				 
         case 'c':
           arg = va_arg(ap, u32);
           USB_UART_putchar(arg);
           break;
         case 'x':
         case 'X':
           arg = va_arg(ap, u32);
           arg1 = arg;
           arg1 = ((arg >> 4) & 0x0000000F);
           if (arg1 < 10)
               putint(arg1);
             else  
               USB_UART_putchar( arg1 - 10 + 65 );
           arg1 = (arg & 0x0000000F);
           if (arg1 < 10)
               putint(arg1);
             else  
               USB_UART_putchar( arg1 - 10 + 65 );
           break;            
         case '2':
           str++;
           if ((*str == 'x') || (*str == 'X'))
           {
             arg = va_arg(ap, u32);
             arg1 = arg;

             for (k=12; k>=0 ;k-=4)
             {
               arg1 = ((arg >> k) & 0x0000000F);
               if (arg1 < 10)
                  putint(arg1);
                else  
                  USB_UART_putchar( arg1 - 10 + 65 );
             }

             break;           
           }
           else ; //to decide how to treat this case 
         case '4':
           str++;
           if ((*str == 'x') || (*str == 'X'))
           {
             arg = va_arg(ap, u32); // u32 original
						 arg1 = arg;
					
             for (k=28; k>=0 ;k-=4)
             {
               arg1 = ((arg >> k) & 0x0000000F);
               if (arg1 < 10)
                  putint(arg1);
                else  
                  USB_UART_putchar( arg1 - 10 + 65 );
             }
             
           }
           else ; //to decide how to treat this case                       
           break;                 
         default:
           break ;
       }
      }
    str++;   
   }/* end while() */


// !PST
   if (usb_plugged) 
   {  
   	// The USB cable is plugged into the USB-Mini connector in the STRx daugther board 
      
   	u8 b_size = count_char;
   	u8 b_index = 0;
   	u8 actual_input_size;
   	
   	while (b_index < count_char)
   	{
   		if (b_size < VIRTUAL_COM_PORT_DATA_SIZE)
   		{
   		   actual_input_size = b_size;
      	}
      	else
   	   {
   	   	actual_input_size = VIRTUAL_COM_PORT_DATA_SIZE;
   	   	
   	   }
   	   
      	// wait until the data transmission is finished
      	while (GetEPTxStatus(ENDP1) == EP_TX_VALID); //0x30
      	// Write data to the virtual UART through USB port 
      	UserToPMABufferCopy(&(serial_usb_buffer_in[b_index]), ENDP1_TXADDR,actual_input_size);
      	SetEPTxCount(ENDP1,actual_input_size); 
      	SetEPTxValid(ENDP1);
      	
      	b_size = b_size - actual_input_size;
      	b_index= b_index + actual_input_size; 
      }
   }  

   va_end(ap);       
}/* end emberSerialPrintf() */


/*******************************************************************************
* Function Name  : emberSerialWaitSend( (first version: to be reviewed
                   according to the Ember function purpose)
* Description    : waits for all byte to be written out of a port
* Input          : u8 port
* Output         : None
* Return         : None
*******************************************************************************/
void emberSerialWaitSend(u8 port)  
{	
	
   if (usb_plugged)
	{
		/* wait until the data transmission is finished */
      while (GetEPTxStatus(ENDP1) == EP_TX_VALID); //0x ???:to be verified
	}
	else
	{
// !PST 
          while(!(USART_GetFlagStatus(ST_CORE_UART, USART_FLAG_TXE)));
   }
   
}/* end emberSerialWaitSend() */


/*******************************************************************************
* Function Name  : emberSerialReadByte(first version: to be reviewed
                   according to the Ember function purpose)
* Description    :  read a byte over the serial 
* Input          : u8 port
* Output         : None
* Return         : None
*******************************************************************************/
u8 emberSerialReadByte(u8 port, u8 *dataByte)
{
	/* Check if the using the virtual UART through USB or the UART */
	if (usb_plugged)
	{
		/* the USB cable is plugged into the USB-Mini connector in the STR daugther board */
		/* Read the data form the virtual UART through USB port */
		if (count_out > 0) 
  		{
   		 *dataByte = buffer_out[buffer_index++];
    		 count_out--;
    		 if (count_out == 0) // all bytes in buffer_out has been processed 
      		buffer_out_empty = 1; // the buffer_out has been processed   
    		 return 0; // SUCCESS
  		}
  		else
  	 		return 1; // FAILED, RX buffer empty	

	}
	else /* No USB cable plugged: use the UART */
	{
		if (bRByteReady == 1) 
   	{
    	 	*dataByte = bRByte;
     		bRByte = '\0';
     		bRByteReady = 0;
     		return 0; //EMBER_SUCCESS 
  	 	}
   	else
     		return 1; //EMBER_SERIAL_RX_EMPTY 
     
   }  /* end if (usb_plugged) */
}/* end emberSerialReadByte() */

/******************* (C) COPYRIGHT 2006 STMicroelectronics *****END OF FILE****/
