/**
  ******************************************************************************
  * @file FLASH_EraseBlock\main.c
  * @brief This file contains the main function for FLASH erase block example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"

/**
  * @addtogroup FLASH_EraseBlock
  * @{
  */

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define LEDS_PORT (GPIOH)
#define LED1_PIN  (GPIO_PIN_3)
#define LED2_PIN  (GPIO_PIN_2)
#define LED3_PIN  (GPIO_PIN_1)
#define LED4_PIN  (GPIO_PIN_0)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
int _fctcpy(char name);
/* Private functions ---------------------------------------------------------*/
void Delay(u16 nCount)
{
    /* Decrement nCount value */
    while (nCount != 0)
    {
        nCount--;
    }
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief How to Write byte / Write word / Erase a block on FLASH memory.
  * @par Examples description
  * - Program 28 bytes from address 0x4000 up to 0x4019
  * - Check programmed bytes
  * - Program 25 words from 0x401A up to 407F
  * - Checked programmed words
  * - Erase the block 0 (0x4000 < address < 0x407F)
  * - Check erase
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - FLASH_SetProgrammingTime()
  * - FLASH_ProgramByte()
  * - FLASH_ProgramWord()
  * - FLASH_EraseBlock()
  * - FLASH_ReadByte()
  */
void main(void)
{
    u32 add, start_add, stop_add, new_val2 =0;
    u8 new_val1, val =0;
    u16 block =0;
    u8 i =0;

    _fctcpy('b');
    /* Initialize I/Os in Output Mode */
    GPIO_Init(LEDS_PORT, LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN,GPIO_MODE_OUT_PP_LOW_FAST);

    /* Define flash programming Time */
    FLASH_SetProgrammingTime(FLASH_PROGRAMTIME_STANDARD);

    /* Unlock Program & Data memories */
    FLASH_Unlock(FLASH_MEMTYPE_DATA);
    FLASH_Unlock(FLASH_MEMTYPE_PROG);
    /* Program 28 bytes & check */
    start_add = 0x4000;
    stop_add = 0x401B;
    new_val1 = 0x55;
    for (add = start_add; add <= stop_add; add++)
            {
                FLASH_ProgramByte(add, new_val1);
                FLASH_WaitForLastOperation();
                if (FLASH_ReadByte(add) != new_val1)
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }
            }

    /* Program 25 words & check */
    start_add = 0x401C;
    stop_add = 0x407F;
    new_val2 = 0xAAAAAAAA;
    add = start_add;
    for (i = 0; i < 25; i++)
    {
        FLASH_ProgramWord(add, new_val2);
        FLASH_WaitForLastOperation();
        add = add + 4;
    }

    /* check */
    start_add = 0x401C;
    for (add = start_add; add <= stop_add; add++)
            {

                val = FLASH_ReadByte(add);
                if (val != BYTE_0(new_val2))
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }

                add += 1;
                val = FLASH_ReadByte(add);
                if (val != BYTE_1(new_val2))
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }

                add += 1;
                val = FLASH_ReadByte(add);
                if (val != BYTE_2(new_val2))
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }

                add += 1;
                val = FLASH_ReadByte(add);
                if (val != BYTE_3(new_val2))
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }

            }

    /* Erase block0 & check */
    block = 0; /*block 0 is first block of Data memory: address is 4000*/
    start_add = 0x4000;
    stop_add = 0x407F;
    FLASH_EraseBlock(block, FLASH_MEMTYPE_DATA);
    FLASH_WaitForLastOperation();
    for (add = start_add; add <= stop_add; add++)
            {
                if (FLASH_ReadByte(add) != 0x00)
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }
            }

    /* Pass */
    while (1)
    {
        GPIO_WriteReverse(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));
        Delay((u16)60000);
        Delay((u16)60000);
    }

}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
    /* Infinite loop */
    while (1)
    {
    }
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
