/**
  ******************************************************************************
  * @file FLASH_WriteEraseBlock\main.c
  * @brief This file contains the main function for FLASH write erase block example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"

/**
  * @addtogroup FLASH_WriteEraseBlock
  * @{
  */

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
#define LEDS_PORT (GPIOH)
#define LED1_PIN  (GPIO_PIN_3)
#define LED2_PIN  (GPIO_PIN_2)
#define LED3_PIN  (GPIO_PIN_1)
#define LED4_PIN  (GPIO_PIN_0)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
int _fctcpy(char name);
/* Private functions ---------------------------------------------------------*/
void Delay(u16 nCount)
{
    /* Decrement nCount value */
    while (nCount != 0)
    {
        nCount--;
    }
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief How to Write / Erase a block on FLASH memory.
  * @par Examples description
  * - Program a block at address 0x4000
  * - Check program using Read byte
  * - Erase block
  * - Check erase with Read byte function.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - FLASH_SetProgrammingTime()
  * - FLASH_ProgramBlock()
  * - FLASH_EraseBlock()
  * - FLASH_ReadByte()
  */
void main(void)
{
    u32 add, start_add, stop_add =0;
    u8 GBuffer[FLASH_BLOCK_SIZE];
    u8 new_val2 = 0xAA;
    u8 block, i =0;

    _fctcpy('b');
    /* Initialize I/Os in Output Mode */
    GPIO_Init(LEDS_PORT, LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN,GPIO_MODE_OUT_PP_LOW_FAST);

    /* Define flash programming Time*/
    FLASH_SetProgrammingTime(FLASH_PROGRAMTIME_STANDARD);

    /* Unlock Program & Data memories */
    FLASH_Unlock(FLASH_MEMTYPE_DATA);
    FLASH_Unlock(FLASH_MEMTYPE_PROG);

    /* Fill the buffer in RAM */
    for (i = 0; i < FLASH_BLOCK_SIZE; i++) GBuffer[i] = new_val2;

    /* Program the block 0*/
    block = 0; /*block 0 is first block of Data memory: address is 4000*/
    FLASH_ProgramBlock(block, FLASH_MEMTYPE_DATA, FLASH_PROGRAMMODE_STANDARD, GBuffer);
    FLASH_WaitForLastOperation();

    /* Check the programmed block*/
    start_add = 0x4000;
    stop_add = 0x407F;
    for (add = start_add; add <= stop_add; add++)
            {
                if (FLASH_ReadByte(add) != new_val2)
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }
            }

    /* Erase block 0 and verify it */
    FLASH_EraseBlock(block, FLASH_MEMTYPE_DATA);
    FLASH_WaitForLastOperation();

    for (add = start_add; add <= stop_add; add++)
            {
                if (FLASH_ReadByte(add) != 0x00)
                {
                    while (1) /* Error */
                    {
                        GPIO_WriteReverse(LEDS_PORT, LED2_PIN);
                        Delay((u16)60000);
                        Delay((u16)60000);
                    }
                }
            }

    /* Pass */
    while (1)
    {
        GPIO_WriteReverse(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));
        Delay((u16)60000);
        Delay((u16)60000);
    }

}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
    /* Infinite loop */
    while (1)
    {
    }
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
