/**
  ******************************************************************************
  * @file FLASH_WriteWord\main.c
  * @brief This file contains the main function for FLASH write word example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"

/**
  * @addtogroup FLASH_WriteWord
  * @{
  */

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
#define LEDS_PORT (GPIOH)
#define LED1_PIN  (GPIO_PIN_3)
#define LED2_PIN  (GPIO_PIN_2)
#define LED3_PIN  (GPIO_PIN_1)
#define LED4_PIN  (GPIO_PIN_0)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
void Delay(u16 nCount)
{
    /* Decrement nCount value */
    while (nCount != 0)
    {
        nCount--;
    }
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief How to Write a word & Erase Byte on FLASH memory.
  * @par Examples description
  * - Program a word at address 0x4100
  * - Check program using Read byte
  * - Erase word using erase byte
  * - Check the 4 bytes (word)  value is 0x00.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - FLASH_SetProgrammingTime()
  * - FLASH_ProgramWord()
  * - FLASH_ReadByte()
  * - FLASH_EraseByte()
  */
void main(void)
{
    u32 start_add = 0x4100;
    u32 add;
    u32 new_val = 0x01234567;
    u8 sub_test_count, val;

    /* Initialize I/Os in Output Mode */
    GPIO_Init(LEDS_PORT, LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN,GPIO_MODE_OUT_PP_LOW_FAST);

    /*Define FLASH programming time*/
    FLASH_SetProgrammingTime(FLASH_PROGRAMTIME_STANDARD);
    /* Unlock Program & Data memories */
    FLASH_Unlock(FLASH_MEMTYPE_DATA);
    FLASH_Unlock(FLASH_MEMTYPE_PROG);
		
    /* Program word at addres 0x4100*/
    FLASH_ProgramWord(start_add, new_val);
		FLASH_WaitForLastOperation();

    /* Check program word action */
    val = FLASH_ReadByte(start_add);
    add = start_add;
    if (val != BYTE_0(new_val))
    {
        while (1) /* Error */
        {
            GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
            Delay((u16)60000);
            Delay((u16)60000);
        }
    }

    add += 1;
    val = FLASH_ReadByte(add);
    if (val != BYTE_1(new_val))
    {
        while (1) /* Error */
        {
            GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
            Delay((u16)60000);
            Delay((u16)60000);
        }
    }

    add += 1;
    val = FLASH_ReadByte(add);
    if (val != BYTE_2(new_val))
    {
        while (1) /* Error */
        {
            GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
            Delay((u16)60000);
            Delay((u16)60000);
        }
    }

    add += 1;
    val = FLASH_ReadByte(add);
    if (val != BYTE_3(new_val))
    {
        while (1) /* Error */
        {
            GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
            Delay((u16)60000);
            Delay((u16)60000);
        }
    }

    /* Erase word using Erase byte */
    add = start_add;
    for (sub_test_count = 0; sub_test_count < 4; sub_test_count++)
    {
        FLASH_EraseByte(add);
				FLASH_WaitForLastOperation();
        add += 1;
    }

    /* Verify Erase action */
    add = start_add;
    for (sub_test_count = 0; sub_test_count < 4; sub_test_count++)
    {
        val = FLASH_ReadByte(add);
        if (val != 0x00)
        {
            while (1) /* Error */
            {
                GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                Delay((u16)60000);
                Delay((u16)60000);
            }
        }
        add += 1;
    }

    /* Pass */
    while (1)
    {
        GPIO_WriteReverse(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));
        Delay((u16)60000);
        Delay((u16)60000);
    }

}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
    /* Infinite loop */
    while (1)
    {
    }
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
