/**
  ******************************************************************************
  * @file I2C_EEPROMReadWrite\main.c
  * @brief This file contains the main function for I2C EEPROM Read Write example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"

/* Evalboard I/Os configuration */

#define LEDS_PORT (GPIOH)
#define LED1_PIN  (GPIO_PIN_3)
#define LED2_PIN  (GPIO_PIN_2)
#define LED3_PIN  (GPIO_PIN_1)
#define LED4_PIN  (GPIO_PIN_0)

/**
  * @addtogroup I2C_EEPROMReadWrite
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
void Delay (u16 nCount);

/* Private functions ---------------------------------------------------------*/

void Delay(u16 nCount)
{
    /* Decrement nCount value */
    while (nCount != 0)
    {
        nCount--;
    }
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief How to make  a write followed by a read in the EPROM.
  * @par Examples description
  * - Read 28 bytes on the devive  (EPROM) 0xA0.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - GPIO_Init
  * - GPIO_WriteHigh
  * - GPIO_WriteReverse
  * - I2C_GenerateSTART
  * - I2C_GenerateSTOP
  * - I2C_DeInit
  * - I2C_Send7bitAddress
  * - I2C_SendData
  * - I2C_CheckEvent
  * - I2C_ClearFlag
  * - I2C_ReceiveData
  * - I2C_AcknowledgeConfig
  */
void main(void)
{

    u8 i = 0;
    u8 data[8] = {0,0,0,0,0,0,0,0};
    u8 Buffer_size = 8;
    u8 Buffer[8] = {0x04, 0x42, 0x53, 0x38, 0x97, 0x03, 0x09, 0x77}; /* Using a 2ko E you cannot program a buff > 8 bytes in write page mode */
    u16 Base_Add = 0x0000;

    /* Initialize I/Os in Output Mode */
    GPIO_Init(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN), GPIO_MODE_OUT_PP_LOW_FAST);

    /* Optional: put here as example only */
    I2C_DeInit();

    /* Initialize the I2C */
    I2C_Init(400000, 0xA0, I2C_DUTYCYCLE_2, I2C_ACK_CURR, I2C_ADDMODE_7BIT, 50);


    /* All LEDs are ON per default */
    GPIO_WriteHigh(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));

    /*** WRITE SEQUENCE ***/
    for (i =Buffer_size; i > 0; i--)
    {

        /* Generate start & wait event detection */
        I2C_GenerateSTART(ENABLE);
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_START_SENT));

        /* Send slave Address in WRITE direction & wait detection event */
        I2C_Send7bitAddress(0xA0, I2C_DIRECTION_TX);
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_ADDRESS_ACKED));
        I2C_ClearFlag(I2C_FLAG_ADDRESSSENTMATCHED);

        /* Send Address (on 2 bytes) of first byte to be written & wait event detection */
        I2C_SendData((u8)(Base_Add >> 8)); /* MSB */
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_TRANSMITTED));
        I2C_SendData((u8)((u8)(Base_Add) + i)); /* LSB */
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_TRANSMITTED));

        /* Send Data to write */
        I2C_SendData(Buffer[i]);
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_TRANSMITTED));

        I2C_GenerateSTOP(ENABLE);
        Delay(1000); /* To let eeprom the time to finish the write operation */

    }

    /***  READ SEQUENCE ***/

    /* Generate start & wait event detection */
    I2C_GenerateSTART(ENABLE);
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_START_SENT));

    /* Send slave Address in write direction & wait detection event */
    I2C_Send7bitAddress(0xA0, I2C_DIRECTION_TX);
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_ADDRESS_ACKED));
    I2C_ClearFlag(I2C_FLAG_ADDRESSSENTMATCHED);

    /* Send Address of first byte to be read & wait event detection */
    I2C_SendData((u8)(Base_Add >> 8)); /* MSB */
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    I2C_SendData((u8)(Base_Add)); /* LSB */
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_TRANSMITTED));

    /* Not mandatory if you want to use multi start mode */
    /* I2C_GenerateSTOP(ENABLE); */

    /* Generate start & wait event detection */
    I2C_GenerateSTART(ENABLE);
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_START_SENT));

    /* Send slave Address in read direction & wait event */
    I2C_Send7bitAddress(0xA0, I2C_DIRECTION_RX);
    while (!I2C_CheckEvent(I2C_EVENT_MASTER_ADDRESS_ACKED));
    I2C_ClearFlag(I2C_FLAG_ADDRESSSENTMATCHED);

    /* Receive data - length is the number of data wanted to be read */
    for (i =Buffer_size; i > 0; i--)
    {
        /* waiting for byte from slave */
        while (!I2C_CheckEvent(I2C_EVENT_MASTER_BYTE_RECEIVED));
        data[i] = (u8)(I2C_ReceiveData());
    }

    /* Need to stop ACK */
    I2C_AcknowledgeConfig(I2C_ACK_NONE);

    /* Send STOP Condition to end transmition*/
    I2C_GenerateSTOP(ENABLE);

    /*** Check transmitted and received buffers ***/
    for (i =Buffer_size; i > 0; i--)
    {
        if (Buffer[i] != data[i])
        {
            /* Toggle only LED1 if error is detected */
            while (1)
            {
                GPIO_WriteReverse(LEDS_PORT, LED1_PIN);
                Delay((u16)60000);
            }
        }
    }

    /* Toggle all LEDs when I2C communication is terminated and buffers are ok */
    while (1)
    {
        GPIO_WriteReverse(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));
        Delay((u16)60000);
    }

}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
    /* Put enter your own code to manage an assert error */
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
