/**
  ******************************************************************************
  * @file I2C_IAP\main.c
  * @brief This file contains the main function for I2C IAP example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"
#include "mono_lcd.h"

/* Evalboard I/Os configuration */

#define LEDS_PORT (GPIOH)
#define LED1_PIN  (GPIO_PIN_3)
#define LED2_PIN  (GPIO_PIN_2)
#define LED3_PIN  (GPIO_PIN_1)
#define LED4_PIN  (GPIO_PIN_0)

/**
  * @addtogroup I2C_IAP
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile u8 Nb_Write;
volatile BoolT READ;
volatile BoolT WRITE;
volatile BoolT STOP;
volatile BoolT BYTE_TX;
volatile BoolT LCD_READ;
volatile BoolT LCD_WRITE;
volatile BoolT XFER_END;
u32 Start_add_RW;
u8 I2C_Buffer_Rx[131];
u8 NbDataInBuffer;
u8 IndexData;


/* Private function prototypes -----------------------------------------------*/
void Delay (u16 nCount);

/* Private functions ---------------------------------------------------------*/

void Delay(u16 nCount)
{
    /* Decrement nCount value */
    while (nCount != 0)
    {
        nCount--;
    }
}

/* Public functions ----------------------------------------------------------*/


/**
  * @brief  IAP I2C communication
  * @par This example provides a short description of how
  * to make  IAP I2C communication an using a programmer
 * (Aardvark module) and a SerialEEPROM (STM8S20x).
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - GPIO_Init
  * - I2C_DeInit
  * - I2C_Init
  * - I2C_ITConfig
  * - GPIO_WriteReverse
  * -SPI_DeInit
  * - SPI_Init
  * - SPI_Cmd
  * - FLASH_Unlock
  * - GPIO_WriteHigh
  * - I2C_ClearFlag
  * - FLASH_ProgramByte
  * - I2C_SendData
  * - I2C_CheckEvent
  * - I2C_ClearITPendingBit
  * - I2C_ReceiveData
  */
void main(void)
{

    READ  				 = FALSE;
    WRITE 				 = FALSE;
    STOP  			 	 = FALSE;
    BYTE_TX 		 = FALSE;
    Start_add_RW   = 0x9000;
    NbDataInBuffer = 0;
    Nb_Write 			 = 0;

    /* Initialize I/Os in Output Mode */
    GPIO_Init(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN), GPIO_MODE_OUT_PP_LOW_FAST);

    /* Optional: put here as example only */
    I2C_DeInit();

    /* Initialize the I2C */
    I2C_Init(400000, 0xA2,I2C_DUTYCYCLE_2, I2C_ACK_CURR, I2C_ADDMODE_7BIT, 10);


    /* Configuration of interrupts only */
    I2C_ITConfig(I2C_IT_EVT, ENABLE);
    enableInterrupts();

    /* Initialize SPI for LCD */
    SPI_DeInit();

    SPI_Init(SPI_FIRSTBIT_MSB, SPI_BAUDRATEPRESCALER_128, SPI_MODE_MASTER, SPI_CLOCKPOLARITY_HIGH, SPI_CLOCKPHASE_2EDGE, SPI_DATADIRECTION_2LINES_FULLDUPLEX, SPI_NSS_SOFT, 0x07);
    SPI_Cmd(ENABLE);

    /*** Unlock FLASH ***/
    FLASH_Unlock(FLASH_MEMTYPE_PROG);
    FLASH_Unlock(FLASH_MEMTYPE_DATA);

    /*** LCD ***/
    LCD_Init();
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "  *** STM8 *** ");
    LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "  IAP-I2C Demo ");

    /* All LEDs are ON per default */
    GPIO_WriteHigh(LEDS_PORT, (LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN));

    while (1)
    {

        /* Init start address to 0x9300 */
        if (Start_add_RW == 0x027FFF)
            Start_add_RW = 0x9300;

        /* Write operations */
        if (STOP == TRUE)
        {
            STOP = FALSE;
            I2C_ClearFlag(I2C_FLAG_STOPDETECTION);
            if ((WRITE == TRUE) && (Nb_Write != 0))
            {
                /* Current Write : No add specified*/
                if (Nb_Write == 1)
                {
                    for (IndexData = 0; IndexData < NbDataInBuffer; IndexData++)
                    {
                        FLASH_ProgramByte(Start_add_RW++, I2C_Buffer_Rx[IndexData]);
                    }
                }
                /*Random or Address Write : add specified in 3 first byte of I2C_Buffer_Rx */
                else
                {
                    Start_add_RW = (u32)((u32)I2C_Buffer_Rx[2] + ((u32)I2C_Buffer_Rx[1] << 8) + ((u32)I2C_Buffer_Rx[0] << 16));
                    for (IndexData = 3; IndexData < NbDataInBuffer; IndexData++)
                    {
                        FLASH_ProgramByte(Start_add_RW++, I2C_Buffer_Rx[IndexData]);
                    }
                }
                LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "Write Done !");
            }

            STOP = FALSE;
            READ = FALSE;
            WRITE = FALSE;
            BYTE_TX = FALSE;
            NbDataInBuffer = 0;
            IndexData = 0;
            Nb_Write = 0;
        }

        /* Read operations */
        if ((READ == TRUE) && (STOP == FALSE))
        {
            if (Nb_Write == 1)
            {
                Start_add_RW = (u32)((u32)I2C_Buffer_Rx[2] + ((u32)I2C_Buffer_Rx[1] << 8) + ((u32)I2C_Buffer_Rx[0] << 16));
                Nb_Write = 0;
            }

            while (BYTE_TX == TRUE)
            {
                BYTE_TX = FALSE; /*Reset the Flag */
                I2C_SendData(FLASH_ReadByte(Start_add_RW++));
            }
        }

        /* The following line-code are implemented to display on the LCD */
        /* You can remove or comment without any impact on IAP I2C */
        if (LCD_WRITE == TRUE)
        {
            LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "  IAP-I2C Demo ");
            LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "Writing ...");
            LCD_WRITE = FALSE;
        }
        if (LCD_READ == TRUE)
        {
            LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "  IAP-I2C Demo ");
            LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "Reading ...");
            LCD_READ = FALSE;
        }
        if ((XFER_END == TRUE) || (I2C_SR3_TRA == (I2C->SR3 & I2C_SR3_TRA)))
        {
            LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "  IAP-I2C Demo ");
            LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "Read Done !");
        }

    }	/* end of While */


}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
    while(1)
		{
		}
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
