/**
  ******************************************************************************
  * @file UART2_Interrupt\main.c
  * @brief This file contains the main function for a basic communication between UART2 and UART1 using interrupts.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_lib.h"
#include "main.h"

/**
  * @addtogroup UART2_Interrupt
  * @{
  */
/* Private typedef -----------------------------------------------------------*/
typedef enum { FAILED = 0, PASSED = !FAILED} TestStatus;
/* Private variables ---------------------------------------------------------*/
vu8 TxBuffer1[] = "UART2 Interrupt Example: UART2 -> UART1 using Interrupt";
vu8 TxBuffer2[] = "UART2 Interrupt Example: UART1 -> UART2 using Interrupt";  
u8 RxBuffer1[RxBufferSize1];
u8 RxBuffer2[RxBufferSize2];
vu8 TxCounter1 = 0x00;
vu8 TxCounter2 = 0x00;
vu8 RxCounter1 = 0x00;
vu8 RxCounter2 = 0x00;
u8 NbrOfDataToTransfer1 = TxBufferSize1;
u8 NbrOfDataToTransfer2 = TxBufferSize2;
u8 NbrOfDataToRead1 = RxBufferSize1;
u8 NbrOfDataToRead2 = RxBufferSize2;
volatile TestStatus TransferStatus1 = FAILED;
volatile TestStatus TransferStatus2 = FAILED;

/* Private function prototypes -----------------------------------------------*/
TestStatus Buffercmp(u8* pBuffer1, u8* pBuffer2, u16 BufferLength);

/* Private functions ---------------------------------------------------------*/
/**
  * @brief Example firmware main entry point.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * - UART2_DeInit()
  * - UART2_Init()
	* - UART2_Cmd()
  * - UART2_ITConfig()
	* - UART1_DeInit()
  * - UART1_Init()
	* - UART1_Cmd()
  * - UART1_ITConfig()
	* - CLK_HSIPrescalerConfig()
  */
void main(void)
{
    /*High speed internal clock prescaler: 1*/
    CLK_HSIPrescalerConfig(CLK_PRESCALER_HSIDIV1);

    /* Enable general interrupts */
    enableInterrupts();

    /* Deinitializes the UART2 and UART1 peripheral */
    UART2_DeInit();
    UART1_DeInit();
    /* UART2 and UART1 configuration -------------------------------------------------*/
    /* UART2 and UART1 configured as follow:
          - BaudRate = 9600 baud  
          - Word Length = 8 Bits
          - One Stop Bit
          - No parity
          - Receive and transmit enabled
          - UART2 Clock disabled
     */
    /* Configure the UART2 */
    UART2_Init((u32)9600, UART2_WORDLENGTH_8D, UART2_STOPBITS_1, UART2_PARITY_NO, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);

    /* Configure the UART1 */
 		UART1_Init((u32)9600, UART1_WORDLENGTH_8D, UART1_STOPBITS_1, UART1_PARITY_NO, UART1_MODE_TXRX_ENABLE);

    /* Enable UART2 Receive and UART1 Transmit interrupt */

    UART1_ITConfig(UART1_IT_RXNE_ORE, ENABLE);
    UART2_ITConfig(UART2_IT_TXE, ENABLE);

    /* Wait until end of transmission from UART2 to UART1 */
    while (GetVar_RxCounter2() < GetVar_NbrOfDataToTransfer1())
    {
    }
    /* Enable UART1 Receive and UART2 Transmit interrupt */
    UART2_ITConfig(UART2_IT_RXNE_ORE, ENABLE);
    UART1_ITConfig(UART1_IT_TXE, ENABLE);
    /* Wait until end of transmission from UART1 to UART2 */
    while (GetVar_RxCounter1() < GetVar_NbrOfDataToTransfer2())
    {
    }

    /* Check the received data with the send ones */
    TransferStatus1 = Buffercmp(TxBuffer2, RxBuffer1, RxBufferSize1);
    /* TransferStatus1 = PASSED, if the data transmitted from UART1 and
       received by UART2 are the same */
    /* TransferStatus1 = FAILED, if the data transmitted from UART1 and
       received by UART2 are different */
    TransferStatus2 = Buffercmp(TxBuffer1, RxBuffer2, RxBufferSize2);
    /* TransferStatus2 = PASSED, if the data transmitted from UART2 and
       received by UART1 are the same */
    /* TransferStatus2 = FAILED, if the data transmitted from UART2 and
       received by UART1 are different */

    while (1);
}

/**
  * @brief Compares two buffers.
  * @param[in] pBuffer1 First buffer to be compared.
  * @param[in] pBuffer2 Second buffer to be compared.
  * @param[in] BufferLength Buffer's length
  * @retval TestStatus Status of buffer comparison
  * - PASSED: pBuffer1 identical to pBuffer2
  * - FAILED: pBuffer1 differs from pBuffer2
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
TestStatus Buffercmp(u8* pBuffer1, u8* pBuffer2, u16 BufferLength)
{
    while (BufferLength--)
    {
        if (*pBuffer1 != *pBuffer2)
        {
            return FAILED;
        }

        pBuffer1++;
        pBuffer2++;
    }

    return PASSED;
}
/**
  * @brief Returns TxCounter1 value.
  * @par Parameters:
  * None
  * @retval Transmit counter index1
  * - u8: TxCounter1
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_TxCounter1(void)
{
return TxCounter1;
}
/**
  * @brief Returns TxCounter2 value.
  * @par Parameters:
  * None
  * @retval Transmit counter index2
  * - u8: TxCounter2
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_TxCounter2(void)
{
return TxCounter2;
}
/**
  * @brief Returns RxCounter1 value.
  * @par Parameters:
  * None
  * @retval Receive counter index1
  * - u8: RxCounter1
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_RxCounter1(void)
{
return RxCounter1;
}
/**
  * @brief Returns RxCounter2 value.
  * @par Parameters:
  * None
  * @retval Receive counter index2
  * - u8: RxCounter2
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_RxCounter2(void)
{
return RxCounter2;
}
/**
  * @brief Increments TxCounter1 variable and return its value 
  * @par Parameters:
  * None
  * @retval Transmit counter index1++
  * - u8: TxCounter1++
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 IncrementVar_TxCounter1(void)
{
	return TxCounter1++;
}
/**
  * @brief Increments TxCounter2 variable and return its value 
  * @par Parameters:
  * None
  * @retval Transmit counter index2++
  * - u8: TxCounter2++
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 IncrementVar_TxCounter2(void)
{
	return TxCounter2++;
}
/**
  * @brief Increments RxCounter1 variable and return its value 
  * @par Parameters:
  * None
  * @retval Receive counter index1++
  * - u8: RxCounter1++
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 IncrementVar_RxCounter1(void)
{
	return RxCounter1++;
}
/**
  * @brief Increments RxCounter2 variable and return its value 
  * @par Parameters:
  * None
  * @retval Receive counter index2++
  * - u8: RxCounter2++
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 IncrementVar_RxCounter2(void)
{
	return RxCounter2++;
}
/**
  * @brief Returns NbrOfDataToTransfer1 value.
  * @par Parameters:
  * None
  * @retval Tx Buffer Size1
  * - u8: NbrOfDataToTransfer1
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_NbrOfDataToTransfer1(void)
{
return NbrOfDataToTransfer1;
}
/**
  * @brief Returns NbrOfDataToTransfer2 value.
  * @par Parameters:
  * None
  * @retval Tx Buffer Size2
  * - u8: NbrOfDataToTransfer2
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_NbrOfDataToTransfer2(void)
{
return NbrOfDataToTransfer2;
}
/**
  * @brief Returns NbrOfDataToRead1 value.
  * @par Parameters:
  * None
  * @retval Rx Buffer Size1
  * - u8: NbrOfDataToRead1
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_NbrOfDataToRead1(void)
{
return NbrOfDataToRead1;
}
/**
  * @brief Returns NbrOfDataToRead2 value.
  * @par Parameters:
  * None
  * @retval Rx Buffer Size2
  * - u8: NbrOfDataToRead2
  * @par Required preconditions:
  * None
  * @par Library functions called:
  * None
  */
u8 GetVar_NbrOfDataToRead2(void)
{
return NbrOfDataToRead2;
}
/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
  /* Infinite loop */
  while (1)
  {
  }
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
