/**
  ******************************************************************************
  * @file stm8s_adc.c
  * @brief This file contains all the functions/macros for the ADC peripheral.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_adc.h"

/* LINKER SECTIONS DEFINITION FOR THIS FILE ONLY */
#ifdef USE_COSMIC_SECTIONS
#pragma section (ADC_CODE)
#pragma section const {ADC_CONST}
#pragma section @near [ADC_URAM]
#pragma section @near {ADC_IRAM}
#pragma section @tiny [ADC_UZRAM]
#pragma section @tiny {ADC_IZRAM}
#endif

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/* Public functions ----------------------------------------------------------*/

/**
  * @addtogroup ADC_Public_Functions
  * @{
  */

/**
  * @brief Deinitializes the ADC peripheral registers to their default reset
  * values.
  * @par Parameters:
  * None
  * @retval None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Initializes ADC to its reset values.
  * @code
  * ADC_DeInit();
  * @endcode
  */
void ADC_DeInit(void)
{
  ADC->CSR  = ADC_CSR_RESET_VALUE;
  ADC->CR1  = ADC_CR1_RESET_VALUE;
  ADC->CR2  = ADC_CR2_RESET_VALUE;
  ADC->CR3  = ADC_CR3_RESET_VALUE;
  ADC->TDRH = ADC_TDRH_RESET_VALUE;
  ADC->TDRL = ADC_TDRL_RESET_VALUE;
  ADC->VRHM = ADC_VRHM_RESET_VALUE;
  ADC->VRHL = ADC_VRHL_RESET_VALUE;
  ADC->VRLM = ADC_VRLM_RESET_VALUE;
  ADC->VRLL = ADC_VRLL_RESET_VALUE;
  ADC->AWENH = ADC_AWENH_RESET_VALUE;
  ADC->AWENL = ADC_AWENL_RESET_VALUE;
}


/**
  * @brief Initializes the ADC peripheral according to the specified parameters
  * @param[in] ADC_ConversionMode: specifies the conversion mode
  * can be one of the values of @ref ADC_ConvMode_TypeDef.
  * @param[in] ADC_Channel: specifies the channel to convert
  * can be one of the values of @ref ADC_Channel_TypeDef.
  * @param[in] ADC_PrescalerSelection: specifies the ADC prescaler
  * can be one of the values of @ref ADC_PresSel_TypeDef.
  * @param[in] ADC_ExtTrigger: specifies the external trigger
  * can be one of the values of @ref ADC_ExtTrig_TypeDef.
  * @param[in] ADC_ExtTrigState: specifies the external trigger new state
  * can be one of the values of @ref FunctionalState.
  * @param[in] ADC_Align: specifies the converted data alignement
  * can be one of the values of @ref ADC_Align_TypeDef.
  * @param[in] ADC_SchmittTriggerChannel: specifies the schmitt trigger channel
  * can be one of the values of @ref ADC_SchmittTrigg_TypeDef.
  * @param[in] ADC_SchmittTriggerState: specifies the schmitt trigger state
  * can be one of the values of @ref FunctionalState.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - ADC_ConversionConfig()
  * - ADC_PrescalerConfig()
  * - ADC_ExternalTriggerConfig()
  * - ADC_SchmittTriggerConfig()
  * - ADC_Cmd()
  * @par Example:
  * Initializes ADC
  * @code
  * ADC_Init(ADC_CONVERSIONMODE_SINGLE, ADC_CHANNEL_2, ADC_PRESSEL_FCPU_D8, ADC_EXTTRIG_TIM, DISABLE, ADC_ALIGN_RIGHT, ADC_SCHMITTTRIG_ALL, DISABLE);
  * @endcode
  */
void ADC_Init(ADC_ConvMode_TypeDef ADC_ConversionMode, ADC_Channel_TypeDef ADC_Channel, ADC_PresSel_TypeDef ADC_PrescalerSelection, ADC_ExtTrig_TypeDef ADC_ExtTrigger, FunctionalState ADC_ExtTrigState, ADC_Align_TypeDef ADC_Align, ADC_SchmittTrigg_TypeDef ADC_SchmittTriggerChannel, FunctionalState ADC_SchmittTriggerState)
{

  /* Check the parameters */
  assert_param(IS_ADC_CONVERSIONMODE_OK(ADC_ConversionMode));
  assert_param(IS_ADC_CHANNEL_OK(ADC_Channel));
  assert_param(IS_ADC_PRESSEL_OK(ADC_PrescalerSelection));
  assert_param(IS_ADC_EXTTRIG_OK(ADC_ExtTrigger));
  assert_param(IS_FUNCTIONALSTATE_OK(((ADC_ExtTrigState))));
  assert_param(IS_ADC_ALIGN_OK(ADC_Align));
  assert_param(IS_ADC_SCHMITTTRIG_OK(ADC_SchmittTriggerChannel));
  assert_param(IS_FUNCTIONALSTATE_OK(ADC_SchmittTriggerState));

  /*-----------------CR1 & CSR configuration --------------------*/
  /* Configure the conversion mode and the channel to convert
  respectively according to ADC_ConversionMode & ADC_Channel values  &  ADC_Align values */
  ADC_ConversionConfig(ADC_ConversionMode, ADC_Channel, ADC_Align);
  /* Select the prescaler division factor according to ADC_PrescalerSelection values */
  ADC_PrescalerConfig(ADC_PrescalerSelection);

  /*-----------------CR2 configuration --------------------*/
  /* Configure the external trigger state and event respectively
  according to ADC_ExtTrigStatus, ADC_ExtTrigger */
  ADC_ExternalTriggerConfig(ADC_ExtTrigger, ADC_ExtTrigState);

  /*------------------TDR configuration ---------------------------*/
  /* Configure the schmitt trigger channel and state respectively
  according to ADC_SchmittTriggerChannel & ADC_SchmittTriggerNewState  values */
  ADC_SchmittTriggerConfig(ADC_SchmittTriggerChannel, ADC_SchmittTriggerState);

  /* Enable the ADC peripheral */
  ADC->CR1 |= ADC_CR1_ADON;

}


/**
  * @brief Enables or Disables the ADC peripheral.
  * @param[in] NewState: specifies the peripheral enabled or disabled state.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable the ADC peripheral.
  * @code
  * ADC_Cmd(ENABLE);
  * @endcode
  */
void ADC_Cmd(FunctionalState NewState)
{

  /* Check the parameters */
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  if (NewState != DISABLE)
  {
    ADC->CR1 |= ADC_CR1_ADON;
  }
  else /* NewState == DISABLE */
  {
    ADC->CR1 &= (u8)(~ADC_CR1_ADON);
  }

}

/**
  * @brief Enables or Disables the ADC scan mode.
  * @param[in] NewState: specifies the selected mode enabled or disabled state.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable the ADC scan mode.
  * @code
  * ADC_ScanModeCmd(ENABLE);
  * @endcode
  */
void ADC_ScanModeCmd(FunctionalState NewState)
{

  /* Check the parameters */
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  if (NewState != DISABLE)
  {
    ADC->CR2 |= ADC_CR2_SCAN;
  }
  else /* NewState == DISABLE */
  {
    ADC->CR2 &= (u8)(~ADC_CR2_SCAN);
  }

}

/**
  * @brief Enables or Disables the ADC data store into the Data Buffer registers rather than in the Data Register
  * @param[in] NewState: specifies the selected mode enabled or disabled state.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable the ADC data buffer.
  * @code
  * ADC_DataBufferCmd(ENABLE);
  * @endcode
  */
void ADC_DataBufferCmd(FunctionalState NewState)
{

  /* Check the parameters */
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  if (NewState != DISABLE)
  {
    ADC->CR3 |= ADC_CR3_DBUF;
  }
  else /* NewState == DISABLE */
  {
    ADC->CR3 &= (u8)(~ADC_CR3_DBUF);
  }

}

/**
  * @brief Enables or disables the ADC interrupt.
  * @param[in] ADC_IT specifies the name of the interrupt to enable or disable.
  * This parameter can be one of the following values:
  *    - ADC_IT_AWDGITEN : Analog WDG interrupt enable
  *    - ADC_IT_EOCITEN  : EOC iterrupt enable
  * @param[in] ADC_ITEnable specifies the state of the interrupt to apply.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable the ADC End Of Convertion (EOC) Interrupt.
  * @code
  * ADC_ITConfig(ADC_IT_EOCITEN, ENABLE);
  * @endcode
  */
void ADC_ITConfig(ADC_IT_TypeDef ADC_IT, FunctionalState ADC_ITEnable)
{

  /* Check the parameters */
  assert_param(IS_ADC_IT_OK(ADC_IT));
  assert_param(IS_FUNCTIONALSTATE_OK(ADC_ITEnable));

  if (ADC_ITEnable != DISABLE)
  {
    /* Enable the ADC interrupts */
    ADC->CSR |= ADC_IT;
  }
  else  /*ADC_ITEnable == DISABLE */
  {
    /* Disable the ADC interrupts */
    ADC->CSR &= (u8)(~ADC_IT);
  }

}

/**
  * @brief Configure the ADC prescaler division factor.
  * @param[in] ADC_Prescaler: the selected precaler.
  * It can be one of the values of @ref ADC_PresSel_TypeDef.
  * @retval void None
  * @par Required preconditions:
  *  None
  * @par Called functions:
  * None
  * @par Example:
  * Configure the prescaler with the value of fadc = fcpu/4
  * @code
  * ADC_PrescalerConfig(ADC_PRESSEL_FCPU_D4 );
  * @endcode
  */
void ADC_PrescalerConfig(ADC_PresSel_TypeDef ADC_Prescaler)
{

  /* Check the parameter */
  assert_param(IS_ADC_PRESSEL_OK(ADC_Prescaler));

  /* Clear the SPSEL bits */
  ADC->CR1 &= (u8)(~ADC_CR1_SPSEL);
  /* Select the prescaler division factor according to ADC_PrescalerSelection values */
  ADC->CR1 |= (u8)(ADC_Prescaler);

}


/**
  * @brief Enables or disables the ADC Schmitt Trigger on a selected channel.
  * @param[in] ADC_SchmittTriggerChannel specifies the desired Channel.
  * It can be set of the values of @ref ADC_SchmittTrigg_TypeDef.
  * @param[in] ADC_SchmittTriggerState specifies Channel new status.
  * can have one of the values of @ref FunctionalState.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enables the schmitt trigger on the channel 8.
  * @code
  * ADC_SchmittTriggerConfig(ADC_SCHMITTTRIG_CHANNEL8,ENABLE);
  * @endcode
  */
void ADC_SchmittTriggerConfig(ADC_SchmittTrigg_TypeDef ADC_SchmittTriggerChannel, FunctionalState ADC_SchmittTriggerState)
{

  /* Check the parameters */
  assert_param(IS_ADC_SCHMITTTRIG_OK(ADC_SchmittTriggerChannel));
  assert_param(IS_FUNCTIONALSTATE_OK(ADC_SchmittTriggerState));

  if (ADC_SchmittTriggerChannel == ADC_SCHMITTTRIG_ALL)
  {
    if (ADC_SchmittTriggerState != DISABLE)
    {
      ADC->TDRL &= (u8)0x0;
      ADC->TDRH &= (u8)0x0;
    }
    else /* ADC_SchmittState == DISABLE */
    {
      ADC->TDRL |= (u8)0xFF;
      ADC->TDRH |= (u8)0xFF;
    }
  }
  else if (ADC_SchmittTriggerChannel < ADC_SCHMITTTRIG_CHANNEL8)
  {
    if (ADC_SchmittTriggerState != DISABLE)
    {
      ADC->TDRL &= (u8)(~(u8)((u8)0x01 << (u8)ADC_SchmittTriggerChannel));
    }
    else /* ADC_SchmittState == DISABLE */
    {
      ADC->TDRL |= (u8)((u8)0x01 << (u8)ADC_SchmittTriggerChannel);
    }
  }
  else /* ADC_SchmittTriggerChannel >= ADC_SCHMITTTRIG_CHANNEL8 */
  {
    if (ADC_SchmittTriggerState != DISABLE)
    {
      ADC->TDRH &= (u8)(~(u8)((u8)0x01 << ((u8)ADC_SchmittTriggerChannel - (u8)8)));
    }
    else /* ADC_SchmittState == DISABLE */
    {
      ADC->TDRH |= (u8)((u8)0x01 << ((u8)ADC_SchmittTriggerChannel - (u8)8));
    }
  }

}


/**
  * @brief Configure the ADC conversion on selected channel.
  * @param[in] ADC_ConversionMode Specifies the conversion type.
  * It can be set of the values of @ref ADC_ConvMode_TypeDef
  * @param[in] ADC_Channel specifies the ADC Channel.
  * It can be set of the values of @ref ADC_Channel_TypeDef
  * @param[in] ADC_Align specifies the conerted data alignement.
  * It can be set of the values of @ref ADC_Align_TypeDef
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Configure the ADC conversion in continuous mode on channel 2
  * @code
  * ADC_ConversionConfig(ADC_CHANNEL_2, ADC_CONVERSIONMODE_CONTINUOUS, ADC_ALIGN_RIGHT);
  * @endcode
  */
void ADC_ConversionConfig(ADC_ConvMode_TypeDef ADC_ConversionMode, ADC_Channel_TypeDef ADC_Channel, ADC_Align_TypeDef ADC_Align)
{

  /* Check the parameters */
  assert_param(IS_ADC_CONVERSIONMODE_OK(ADC_ConversionMode));
  assert_param(IS_ADC_CHANNEL_OK(ADC_Channel));
  assert_param(IS_ADC_ALIGN_OK(ADC_Align));

  /* Clear the align bit */
  ADC->CR2 &= (u8)(~ADC_CR2_ALIGN);
  /* Configure the data alignment */
  ADC->CR2 |= (u8)(ADC_Align);

  if (ADC_ConversionMode == ADC_CONVERSIONMODE_CONTINUOUS)
  {
    /* Set the continuous coversion mode */
    ADC->CR1 |= ADC_CR1_CONT;
  }
  else /* ADC_ConversionMode == ADC_CONVERSIONMODE_SINGLE */
  {
    /* Set the single conversion mode */
    ADC->CR1 &= (u8)(~ADC_CR1_CONT);
  }

  /* Clear the ADC channels */
  ADC->CSR &= (u8)(~ADC_CSR_CH);
  /* Select the ADC channel */
  ADC->CSR |= (u8)(ADC_Channel);

}


/**
  * @brief Configure the ADC conversion on external trigger event.
  * @par Full description:
  * The selected external trigger evant can be enabled or disabled.
  * @param[in] ADC_ExtTrigger to select the External trigger event.
  * can have one of the values of @ref ADC_ExtTrig_TypeDef.
  * @param[in] ADC_ExtTrigState to enable/disable the selected external trigger
  * can have one of the values of @ref FunctionalState.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable the TIM1 TRGO to trigger the conversion.
  * @code
  * ADC_ExternalTriggerConfig(ADC_EXTTRIG_TIM, ENABLE);
  * @endcode
  */
void ADC_ExternalTriggerConfig(ADC_ExtTrig_TypeDef ADC_ExtTrigger, FunctionalState ADC_ExtTrigState)
{

  /* Check the parameters */
  assert_param(IS_ADC_EXTTRIG_OK(ADC_ExtTrigger));
  assert_param(IS_FUNCTIONALSTATE_OK(ADC_ExtTrigState));

  /* Clear the external trigger selection bits */
  ADC->CR2 &= (u8)(~ADC_CR2_EXTSEL);

  if (ADC_ExtTrigState != DISABLE)
  {
    /* Enable the selected external Trigger */
    ADC->CR2 |= (u8)(ADC_CR2_EXTTRIG);
  }
  else /* ADC_ExtTrigStatus == DISABLE */
  {
    /* Disable the selected external trigger */
    ADC->CR2 &= (u8)(~ADC_CR2_EXTTRIG);
  }

  /* Set the slected external trigger */
  ADC->CR2 |= (u8)(ADC_ExtTrigger);

}


/**
  * @brief Start ADC conversion
  * @par Full description:
  * This function  triggers the start of conversion, after ADC configuration.
  * @par Parameters:
  * None
  * @retval void None
  * @par Required preconditions:
  * Enable the ADC peripheral before calling this fuction
  * @par Called functions:
  * None
  * @par Example:
  * Start conversion
  * @code
  * ADC_StartConversion();
  * @endcode
  */
void ADC_StartConversion(void)
{
  ADC->CR1 |= ADC_CR1_ADON;
}

/**
  * @brief Get one sample of measured signal.
  * @par Parameters:
  * None
  * @retval ConversionValue:  value of the measured signal.
  * @par Required preconditions:
  * ADC conversion finished.
  * @par Called functions:
  * None
  * @par Example:
  * Get the converted value of a given signal on a given channel.
  * @code
  * u16 ADC_ConversionValue;
  * ADC_ConversionValue = ADC_GetConversionValue();
  * @endcode
  */
u16 ADC_GetConversionValue(void)
{

  u16 temph = 0;
  u8 templ = 0;

  if (ADC->CR2 & ADC_CR2_ALIGN) /* Right alignment */
  {
    /* Read LSB first */
    templ = ADC->DRL;
    /* Then read MSB */
    temph = ADC->DRH;

    temph = (u16)(templ | (u16)(temph << (u8)8));
  }
  else /* Left alignment */
  {
    /* Read MSB firts*/
    temph = ADC->DRH;
    /* Then read LSB */
    templ = ADC->DRL;

    temph = (u16)((u16)(templ << (u8)6) | (u16)(temph << (u8)8));
  }

  return ((u16)temph);

}

/**
  * @brief Enables or disables the analog watchdog for the given channel.
  * @param[in] Channel specifies the desired Channel.
  * It can be set of the values of @ref ADC_Channel_TypeDef.
  * @param[in] NewState specifies the analog watchdog new state.
  * can have one of the values of @ref FunctionalState.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enables the analog watchdog on the channel 8.
  * @code
  * ADC_AWDGChannelConfig(ADC_CHANNEL_8,ENABLE);
  * @endcode
  */
void ADC_AWDGChannelConfig(ADC_Channel_TypeDef Channel, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));
  assert_param(IS_ADC_CHANNEL_OK(Channel));

  if (Channel < (u8)8)
  {
    if (NewState != DISABLE)
    {
      ADC->AWENL |= (u8)((u8)1 << Channel);
    }
    else /* NewState == DISABLE */
    {
      ADC->AWENL &= (u8)(~((u8)1 << Channel));
    }
  }
  else
  {
    if (NewState != DISABLE)
    {
      ADC->AWENH |= (u8)((u8)1 << (Channel - (u8)8));
    }
    else /* NewState == DISABLE */
    {
      ADC->AWENH &= (u8)(~((u8)1 << (Channel - (u8)8)));
    }
  }
}

/**
  * @brief Sets the high threshold of the analog watchdog.
  * @param[in] Threshold specifies the high threshold value.
  * this value depends on the reference voltage range.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  *Set the high threshold to 0x2FF.
  * @code
  * ADC_SetHighThreshold(0x2FF);
  * @endcode
  */
void ADC_SetHighThreshold(u16 Threshold)
{
  ADC->VRHM = (u8)(Threshold >> (u8)2);
  ADC->VRHL = (u8)Threshold;
}

/**
  * @brief Sets the low threshold of the analog watchdog.
  * @param[in] Threshold specifies thelow threshold value.
  * this value depends on the reference voltage range.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  *Set the low threshold to 0x0FF.
  * @code
  * ADC_SetLowThreshold(0x0FF);
  * @endcode
  */
void ADC_SetLowThreshold(u16 Threshold)
{
  ADC->VRLM = (u8)(Threshold >> (u8)2);
  ADC->VRLL = (u8)Threshold;
}

/**
  * @brief Get one sample of measured signal.
  * @param[in] Buffer specifies the buffer to read.
  * @retval BufferValue:  value read from the given buffer.
  * @par Required preconditions:
  * ADC conversion finished.
  * @par Called functions:
  * None
  * @par Example:
  * Get the converted value of a given signal in the second buffer.
  * @code
  * u16 ADC_BufferValue;
  * ADC_BufferValue = ADC_GetBufferValue(2);
  * @endcode
  */
u16 ADC_GetBufferValue(u8 Buffer)
{

  u16 temph = 0;
  u8 templ = 0;

  /* Check the parameters */
  assert_param(IS_ADC_BUFFER_OK(Buffer));

  if (ADC->CR2 & ADC_CR2_ALIGN) /* Right alignment */
  {
    /* Read LSB first */
    templ = *(u8*)(ADC_BaseAddress + (Buffer << 1) + 1);
    /* Then read MSB */
    temph = *(u8*)(ADC_BaseAddress + (Buffer << 1));

    temph = (u16)(templ | (u16)(temph << (u8)8));
  }
  else /* Left alignment */
  {
    /* Read MSB firts*/
    temph = *(u8*)(ADC_BaseAddress + (Buffer << 1));
    /* Then read LSB */
    templ = *(u8*)(ADC_BaseAddress + (Buffer << 1) + 1);

    temph = (u16)((u16)(templ << (u8)6) | (u16)(temph << (u8)8));
  }

  return ((u16)temph);

}

/**
  * @brief Checks the specified analog watchdog channel status.
  * @param[in] Channel: specify the channel of which to check the analog watchdog
  * can be one of the values of @ref ADC_Channel_TypeDef.
  * @retval FlagStatus Status of the analog watchdog.
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Get the EOC flag status.
  * @code
  * FlagStatus ADC_analogwatchdog;
  * ADC_analogwatchdog = ADC_GetAWDGChannelStatus(ADC_CHANNEL_8);
  * @endcode
  */
FlagStatus ADC_GetAWDGChannelStatus(ADC_Channel_TypeDef Channel)
{
  u8 status = 0;

  /* Check the parameters */
  assert_param(IS_ADC_CHANNEL_OK(Channel));

  if (Channel < (u8)8)
  {
    status = (u8)(ADC->AWL & ((u8)1 << Channel));
  }
  else /* Channel = 8 | 9 */
  {
    status = (u8)(ADC->AWH & ((u8)1 << (Channel - (u8)8)));
  }

  return ((FlagStatus)status);
}

/**
  * @brief Checks the specified ADC flag status.
  * @param[in] Flag: ADC flag.
  * can be one of the values of @ref ADC_Flag_TypeDef.
  * @retval FlagStatus Status of the ADC flag.
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Get the EOC flag status.
  * @code
  * FlagStatus ADC_FlagStatus;
  * ADC_FlagStatus = ADC_GetFlagStatus(ADC_FLAG_EOC);
  * @endcode
  */
FlagStatus ADC_GetFlagStatus(ADC_Flag_TypeDef Flag)
{
  u8 flagstatus = 0;
  u8 temp = 0;

  /* Check the parameters */
  assert_param(IS_ADC_FLAG_OK(Flag));

  if ((Flag & 0x0F) == 0x01)
  {
    /* Get OVR flag status */
    flagstatus = (u8)(ADC->CR3 & ADC_CR3_OVR);
  }
  else if ((Flag & 0xF0) == 0x10)
  {
    /* Get analog watchdog channel status */
    temp = (u8)(Flag & 0x0F);
    if (temp < 8)
    {
      flagstatus = (u8)(ADC->AWL & (1 << temp));
    }
    else
    {
      flagstatus = (u8)(ADC->AWH & (1 << (temp - 8)));
    }
  }
  else  /* Get EOC | AWDG flag status */
  {
    flagstatus = (u8)(ADC->CSR & Flag);
  }
  return ((FlagStatus)flagstatus);

}

/**
  * @brief Clear the specified ADC Flag.
  * @param[in] Flag: ADC flag.
  * can be one of the values of @ref ADC_Flag_TypeDef.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Clears the end of conversion Flag
  * @code
  * ADC_ClearFlag();
  * @endcode
  */
void ADC_ClearFlag(ADC_Flag_TypeDef Flag)
{
  u8 temp = 0;

  /* Check the parameters */
  assert_param(IS_ADC_FLAG_OK(Flag));

  if ((Flag & 0x0F) == 0x01)
  {
    /* Clear OVR flag status */
    ADC->CR3 &= (u8)(~ADC_CR3_OVR);
  }
  else if ((Flag & 0xF0) == 0x10)
  {
    /* Clear analog watchdog channel status */
    temp = (u8)(Flag & 0x0F);
    if (temp < 8)
    {
      ADC->AWL &= (u8)(~((u8)1 << temp));
    }
    else
    {
      ADC->AWH &= (u8)(~((u8)1 << (temp - 8)));
    }
  }
  else  /* Clear EOC | AWDG flag status */
  {
    ADC->CSR &= (u8) (~Flag);
  }
}

/**
  * @brief Returns the specified pending bit status
  * @param[in] ITPendingBit : the IT pending bit to check.
  * This parameter can be one of the following values:
  *    - ADC_IT_AWDG   : Analog WDG IT status
  *    - ADC_IT_AWDGS0 : Analog channel 0 IT status
  *    - ADC_IT_AWDGS1 : Analog channel 1 IT status
  *    - ADC_IT_AWDGS2 : Analog channel 2 IT status
  *    - ADC_IT_AWDGS3 : Analog channel 3 IT status
  *    - ADC_IT_AWDGS4 : Analog channel 4 IT status
  *    - ADC_IT_AWDGS5 : Analog channel 5 IT status
  *    - ADC_IT_AWDGS6 : Analog channel 6 IT status
  *    - ADC_IT_AWDGS7 : Analog channel 7 IT status
  *    - ADC_IT_AWDGS8 : Analog channel 8 IT status
  *    - ADC_IT_AWDGS9 : Analog channel 9 IT status
  *    - ADC_IT_EOC    : EOC pending bit
  * @retval FlagStatus: status of the specified pending bit.
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Get the EOC bit status.
  * @code
  * ITStatus ADC_ITStatus;
  * ADC_ITStatus = ADC_GetITStatus();
  * @endcode
  */
ITStatus ADC_GetITStatus(ADC_IT_TypeDef ITPendingBit)
{
  ITStatus itstatus = RESET;
  u8 temp = 0;

  /* Check the parameters */
  assert_param(IS_ADC_ITPENDINGBIT_OK(ITPendingBit));

  if ((ITPendingBit & 0xF0) == 0x10)
  {
    /* Get analog watchdog channel status */
    temp = (u8)(ITPendingBit & 0x0F);
    if (temp < 8)
    {
      itstatus = (u8)(ADC->AWL & (u8)((u8)1 << temp));
    }
    else
    {
      itstatus = (u8)(ADC->AWH & (u8)((u8)1 << (temp - 8)));
    }
  }
  else  /* Get EOC | AWDG flag status */
  {
    itstatus = (u8)(ADC->CSR & ITPendingBit);
  }
  return ((ITStatus)itstatus);

}

/**
  * @brief Clear the ADC End of Conversion pending bit.
  * @param[in] ITPendingBit : the IT pending bit to clear.
  * This parameter can be one of the following values:
  *    - ADC_IT_AWDG   : Analog WDG IT status
  *    - ADC_IT_AWDGS0 : Analog channel 0 IT status
  *    - ADC_IT_AWDGS1 : Analog channel 1 IT status
  *    - ADC_IT_AWDGS2 : Analog channel 2 IT status
  *    - ADC_IT_AWDGS3 : Analog channel 3 IT status
  *    - ADC_IT_AWDGS4 : Analog channel 4 IT status
  *    - ADC_IT_AWDGS5 : Analog channel 5 IT status
  *    - ADC_IT_AWDGS6 : Analog channel 6 IT status
  *    - ADC_IT_AWDGS7 : Analog channel 7 IT status
  *    - ADC_IT_AWDGS8 : Analog channel 8 IT status
  *    - ADC_IT_AWDGS9 : Analog channel 9 IT status
  *    - ADC_IT_EOC    : EOC pending bit
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Clears the end of conversion pending bit
  * @code
  * ADC_ClearITPendingBit(ADC_ITPENDINGBIT_EOC);
  * @endcode
  */
void ADC_ClearITPendingBit(ADC_IT_TypeDef ITPendingBit)
{
  u8 temp = 0;

  /* Check the parameters */
  assert_param(IS_ADC_ITPENDINGBIT_OK(ITPendingBit));

  if ((ITPendingBit& 0xF0) == 0x10)
  {
    /* Clear analog watchdog channel status */
    temp = (u8)(ITPendingBit & 0x0F);
    if (temp < 8)
    {
      ADC->AWL &= (u8)(~((u8)1 << temp));
    }
    else
    {
      ADC->AWH &= (u8)(~((u8)1 << (temp - 8)));
    }
  }
  else  /* Clear EOC | AWDG flag status */
  {
    ADC->CSR &= (u8) (~ITPendingBit);
  }
}

/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
