/**
  ********************************************************************************
  * @file stm8s_uart1.c
  * @brief This file contains all the functions for the uart1 peripheral.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 18-JUL-2008
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm8s_uart1.h"
#include "stm8s_clk.h"

/* LINKER SECTIONS DEFINITION FOR THIS FILE ONLY */
#ifdef USE_COSMIC_SECTIONS
#pragma section (UART1_CODE)
#pragma section const {UART1_CONST}
#pragma section @near [UART1_URAM]
#pragma section @near {UART1_IRAM}
#pragma section @tiny [UART1_UZRAM]
#pragma section @tiny {UART1_IZRAM}
#endif

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/
/* Public functions ----------------------------------------------------------*/

/** @}
  * @addtogroup UART1_Public_Functions
  * @{
  */

/**
  * @brief Deinitializes the UART1 peripheral.
  * @par Full description:
  * Set the UART1 peripheral registers to their default reset values.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Deinitialize UART1 peripheral.
  * @code
  * UART1_DeInit();
  * @endcode
  */

void UART1_DeInit(void)
{
  u8 dummy = 0;
  /*< Clear the Idle Line Detected bit in the status rerister by a read
     to the UART1_SR register followed by a Read to the UART1_DR register */
  dummy = UART1->SR;
  dummy = UART1->DR;

  UART1->BRR2 = UART1_BRR2_RESET_VALUE;  /*< Set UART1_BRR2 to reset value 0x00 */
  UART1->BRR1 = UART1_BRR1_RESET_VALUE;  /*< Set UART1_BRR1 to reset value 0x00 */

  UART1->CR1 = UART1_CR1_RESET_VALUE; /*< Set UART1_CR1 to reset value 0x00  */
  UART1->CR2 = UART1_CR2_RESET_VALUE; /*< Set UART1_CR2 to reset value 0x00  */
  UART1->CR3 = UART1_CR3_RESET_VALUE;  /*< Set UART1_CR3 to reset value 0x00  */
  UART1->CR4 = UART1_CR4_RESET_VALUE;  /*< Set UART1_CR4 to reset value 0x00  */
  UART1->CR5 = UART1_CR5_RESET_VALUE; /*< Set UART1_CR5 to reset value 0x00  */

}

/**
  * @brief Initializes the UART1 according to the specified parameters.
  * @param[in] BaudRate: The baudrate.
 * @param[in] WordLength : This parameter can be any of the @ref UART1_WordLength_TypeDef enumeration.
  * @param[in] StopBits: This parameter can be any of the @ref UART1_StopBits_TypeDef enumeration.
  * @param[in] Parity: This parameter can be any of the @ref UART1_Parity_TypeDef enumeration.
  * @param[in] Mode: This parameter can be any of the @ref UART1_Mode_TypeDef values
 * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * - CLK_GetClockFreq().
  * @par Example:
  * @code
  * UART1_Init(9600, UART1_WORDLENGTH_8D, UART1_STOPBITS_1, UART1_PARITY_NO, UART1_MODE_TXRX_ENABLE);
  * @endcode
  */
void UART1_Init(u32 BaudRate, UART1_WordLength_TypeDef WordLength, UART1_StopBits_TypeDef StopBits, UART1_Parity_TypeDef Parity, UART1_Mode_TypeDef Mode)
{
  u8 BRR2_1, BRR2_2 = 0;
  u32 BaudRate_Mantissa, BaudRate_Mantissa100 = 0;

  assert_param(IS_UART1_WORDLENGTH_OK(WordLength));

  assert_param(IS_UART1_STOPBITS_OK(StopBits));

  assert_param(IS_UART1_PARITY_OK(Parity));

  /* assert_param: BaudRate value should be <= 625000 bps */
  assert_param(IS_UART1_BAUDRATE_OK(BaudRate));

  /* assert_param: UART1_Mode value should exclude values such as  UART1_ModeTx_Enable|UART1_ModeTx_Disable */
  assert_param(IS_UART1_MODE_OK((u8)Mode));

  /* Wait for no Transmition before modifying the M bit */
  /* while(!(UART1->SR&UART1_SR_TC));      */

  UART1->CR1 &= (u8)(~UART1_CR1_M);     /**< Clear the word length bit */
  UART1->CR1 |= (u8)WordLength; /**< Set the word length bit according to UART1_WordLength value */

  UART1->CR3 &= (u8)(~UART1_CR3_STOP);  /**< Clear the STOP bits */
  UART1->CR3 |= (u8)StopBits;  /**< Set the STOP bits number according to UART1_StopBits value  */

  UART1->CR1 &= (u8)(~(UART1_CR1_PCEN | UART1_CR1_PS));  /**< Clear the Parity Control bit */
  UART1->CR1 |= (u8)Parity;     /**< Set the Parity Control bit to UART1_Parity value */

  UART1->BRR1 &= (u8)(~UART1_BRR1_DIVM);  /**< Clear the LSB mantissa of UART1DIV  */
  UART1->BRR2 &= (u8)(~UART1_BRR2_DIVM);  /**< Clear the MSB mantissa of UART1DIV  */
  UART1->BRR2 &= (u8)(~UART1_BRR2_DIVF);  /**< Clear the Fraction bits of UART1DIV */

  /**< Set the UART1 BaudRates in BRR1 and BRR2 registers according to UART1_BaudRate value */
  BaudRate_Mantissa    = ((u32)CLK_GetClockFreq() / (BaudRate << 4));
  BaudRate_Mantissa100 = (((u32)CLK_GetClockFreq() * 100) / (BaudRate << 4));
  BRR2_1 = (u8)((u8)(((BaudRate_Mantissa100 - (BaudRate_Mantissa * 100))
                      << 4) / 100) & (u8)0x0F); /**< Set the fraction of UART1DIV  */
  BRR2_2 = (u8)((BaudRate_Mantissa >> 4) & (u8)0xF0);

  UART1->BRR2 = (u8)(BRR2_1 | BRR2_2);
  UART1->BRR1 = (u8)BaudRate_Mantissa;           /**< Set the LSB mantissa of UART1DIV  */

  if ((u8)Mode&(u8)UART1_MODE_TX_ENABLE)
  {
    UART1->CR2 |= UART1_CR2_TE;  /**< Set the Transmitter Enable bit */
  }
  else
  {
    UART1->CR2 &= (u8)(~UART1_CR2_TE);  /**< Clear the Transmitter Disable bit */
  }
  if ((u8)Mode & (u8)UART1_MODE_RX_ENABLE)
  {
    UART1->CR2 |= UART1_CR2_RE;  /**< Set the Receiver Enable bit */
  }
  else
  {
    UART1->CR2 &= (u8)(~UART1_CR2_RE);  /**< Clear the Receiver Disable bit */
  }
}
/**
  * @brief Enable the UART1 peripheral.
  * @par Full description:
  * Enable the UART1 peripheral.
  * @param[in] NewState new state of the UART1 Communication.
  * This parameter can be:
  * - ENABLE
  * - DISABLE
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Enable UART1 peripheral.
  * @code
  * UART1_Cmd(ENABLE);
  * @endcode
  */
void UART1_Cmd(FunctionalState NewState)
{

  if (NewState != DISABLE)
  {
    UART1->CR1 &= (u8)(~UART1_CR1_UARTD); /**< UART1 Enable */
  }
  else
  {
    UART1->CR1 |= UART1_CR1_UARTD;  /**< UART1 Disable (for low power consumption) */
  }
}

/**
 * @brief Enables or disables the specified UART1 interrupts.
 * @par Full description:
 * Enables or disables the specified UART1 interrupts.
 * @param[in] UART1_IT specifies the UART1 interrupt sources to be enabled or disabled.
 * This parameter can be one of the following values:
 *   - UART1_IT_LBD:  LIN Break detection interrupt
 *   - UART1_IT_LHDF:  LIN Break detection interrupt
 *   - UART1_IT_TXE:  Tansmit Data Register empty interrupt
 *   - UART1_IT_TC:   Transmission complete interrupt
 *   - UART1_IT_RXNE_ORE: Receive Data register not empty/Over run error interrupt
 *   - UART1_IT_IDLE: Idle line detection interrupt
 *   - UART1_IT_PE:   Parity Error interrupt
 * @param[in] NewState new state of the specified UART1 interrupts.
 * This parameter can be: ENABLE or DISABLE.
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * None
 * @par Example:
 * @code
 * UART1_ITConfig(UART1_IT_PE, ENABLE)
 * @endcode
 */
void UART1_ITConfig(UART1_IT_TypeDef UART1_IT, FunctionalState NewState)
{
  u8 uartreg, itpos = 0x00;
  assert_param(IS_UART1_CONFIG_IT_OK(UART1_IT));
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  /* Get the UART1 register index */
  uartreg = (u8)(UART1_IT >> 0x08);

  /* Get the UART1 IT index */
  itpos = (u8)((u8)1 << (u8)((u8)UART1_IT & (u8)0x0F));

  if (NewState != DISABLE)
  {
    /**< Enable the Interrupt bits according to UART1_IT mask */
    if (uartreg == 0x01)
    {
      UART1->CR1 |= itpos;
    }
    else if (uartreg == 0x02)
    {
      UART1->CR2 |= itpos;
    }
    else if (uartreg == 0x03)
    {
      UART1->CR4 |= itpos;
    }
    else
    {
      UART1->CR5 |= itpos;
    }
  }
  else
  {
    /**< Disable the interrupt bits according to UART1_IT mask */
    if (uartreg == 0x01)
    {
      UART1->CR1 &= (u8)(~itpos);
    }
    else if (uartreg == 0x02)
    {
      UART1->CR2 &= (u8)(~itpos);
    }
    else if (uartreg == 0x03)
    {
      UART1->CR4 &= (u8)(~itpos);
    }
    else
    {
      UART1->CR5 &= (u8)(~itpos);
    }
  }
}

/**
  * @brief Sets the UART1 LIN Break detection length.
  * @par Full description:
  * Sets the UART1 LIN Break detection length.
  * @param[in] UART1_LINBreakDetectionLength specifies the LIN break detection length.
  * This parameter can be any of the @ref UART1_LINBreakDetectionLength_TypeDef values.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * UART1_LINBreakDetectionConfig(UART1_LINBREAKDETECTIONLENGTH_11BITS)
  * @endcode
  */
void UART1_LINBreakDetectionConfig(UART1_LINBreakDetectionLength_TypeDef UART1_LINBreakDetectionLength)
{
  assert_param(IS_UART1_LINBREAKDETECTIONLENGTH_OK(UART1_LINBreakDetectionLength));

  if (UART1_LINBreakDetectionLength != UART1_LINBREAKDETECTIONLENGTH_10BITS)
  {
    UART1->CR4 |= UART1_CR4_LBDL;
  }
  else
  {
    UART1->CR4 &= ((u8)~UART1_CR4_LBDL);
  }
}

/**
  * @brief Configue the UART1 peripheral.
  * @par Full description:
  * Configue the UART1 peripheral.
  * @param[in] UART1_Mode specifies the LIN mode.
  * This parameter can be any of the @ref UART1_LinMode_TypeDef values.
 * @param[in] UART1_Autosync specifies the LIN automatic resynchronization mode.
  * This parameter can be any of the @ref UART1_LinAutosync_TypeDef values.
 * @param[in] UART1_DivUp specifies the LIN divider update method.
  * This parameter can be any of the @ref UART1_LinDivUp_TypeDef values.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * UART1_LINConfig(UART1_LIN_MODE_MASTER, UART1_LIN_AUTOSYNC_DISABLE,UART1_LIN_DIVUP_LBRR1);
  * @endcode
  */
void UART1_LINConfig(UART1_LinMode_TypeDef UART1_Mode, UART1_LinAutosync_TypeDef UART1_Autosync, UART1_LinDivUp_TypeDef UART1_DivUp)
{
  assert_param(IS_UART1_SLAVE_OK(UART1_Mode));

  assert_param(IS_UART1_AUTOSYNC_OK(UART1_Autosync));

  assert_param(IS_UART1_DIVUP_OK(UART1_DivUp));

  if (UART1_Mode != UART1_LIN_MODE_MASTER)
  {
    UART1->CR5 |=  UART1_CR5_LSLV;
  }
  else
  {
    UART1->CR5 &= ((u8)~UART1_CR5_LSLV);
  }

  if (UART1_Autosync != UART1_LIN_AUTOSYNC_DISABLE)
  {
    UART1->CR5 |=  UART1_CR5_LASE ;
  }
  else
  {
    UART1->CR5 &= ((u8)~ UART1_CR5_LASE );
  }

  if (UART1_DivUp != UART1_LIN_DIVUP_LBRR1)
  {
    UART1->CR5 |=  UART1_CR5_LDUM;
  }
  else
  {
    UART1->CR5 &= ((u8)~ UART1_CR5_LDUM);
  }

}

/**
  * @brief Enables or disables the UART1 LIN mode.
  * @par Full description:
  * Enables or disables the UART1s LIN mode.
  * @param[in] NewState is new state of the UART1 LIN mode.
  * This parameter can be:
  * - ENABLE
  * - DISABLE
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * UART1_LINCmd(ENABLE)
  * @endcode
  */
void UART1_LINCmd(FunctionalState NewState)
{
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the LIN mode by setting the LINE bit in the CR2 register */
    UART1->CR3 |= UART1_CR3_LINEN;
  }
  else
  {
    /* Disable the LIN mode by clearing the LINE bit in the CR2 register */
    UART1->CR3 &= ((u8)~UART1_CR3_LINEN);
  }
}

/**
  * @brief Selects the UART1 WakeUp method.
  * @par Full description:
  * Selects the UART1 WakeUp method.
  * @param[in] UART1_WakeUp: specifies the UART1 wakeup method.
  * This parameter can be any of the @ref UART1_WakeUp_TypeDef values.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * UART1 WakeUp Idle line .
  * @code
    * UART1_WakeUpConfig(UART1_WAKEUP_IDLELINE);
  * @endcode
  */
void UART1_WakeUpConfig(UART1_WakeUp_TypeDef UART1_WakeUp)
{
  assert_param(IS_UART1_WAKEUP_OK(UART1_WakeUp));

  UART1->CR1 &= ((u8)~UART1_CR1_WAKE);
  UART1->CR1 |= (u8)UART1_WakeUp;
}

/**
  * @brief Determines if the UART1 is in mute mode or not.
  * @par Full description:
  * Determines if the UART1 is in mute mode or not.
  * @param[in] NewState: new state of the UART1 mode.
  * This parameter can be:
  * - ENABLE
  * - DISABLE
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * UART1_ReceiverWakeUpCmd(DISABLE);
  * @endcode
  */
void UART1_ReceiverWakeUpCmd(FunctionalState NewState)
{
  assert_param(IS_FUNCTIONALSTATE_OK(NewState));

  if (NewState != DISABLE)
  {
    /* Enable the mute mode UART1 by setting the RWU bit in the CR2 register */
    UART1->CR2 |= UART1_CR2_RWU;
  }
  else
  {
    /* Disable the mute mode UART1 by clearing the RWU bit in the CR1 register */
    UART1->CR2 &= ((u8)~UART1_CR2_RWU);
  }
}


/**
  * @brief Returns the most recent received data by the UART1 peripheral.
  * @par Full description:
  * Returns the most recent received data by the UART1 peripheral.
  * @retval u16 Received Data
  * @par Required preconditions:
  * UART1_Cmd(ENABLE);
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * u8 my_data;
  * my_data = UART1_ReceiveData8();
  * @endcode
  */
u8 UART1_ReceiveData8(void)
{
  return ((u8)UART1->DR);
}

/**
  * @brief Returns the most recent received data by the UART1 peripheral.
  * @par Full description:
  * Returns the most recent received data by the UART1 peripheral.
  * @retval u16 Received Data
  * @par Required preconditions:
  * UART1_Cmd(ENABLE);
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * u16 my_data;
  * my_data = UART1_ReceiveData9();
  * @endcode
  */
u16 UART1_ReceiveData9(void)
{
  return (u16)((((u16)UART1->DR) | ((u16)(((u16)((u16)UART1->CR1 & (u16)UART1_CR1_R8)) << 1))) & ((u16)0x01FF));
}



/**
  * @brief Transmits 8 bit data through the UART1 peripheral.
  * @par Full description:
  * Transmits 8 bit data through the UART1 peripheral.
  * @param[in] Data: the data to transmit.
  * @retval void None
  * @par Required preconditions:
  * UART1_Cmd(ENABLE);
  * @par Called functions:
  * None
  * @par Example:
  * Send 0x55 using UART1
  * @code
  * u8 my_data = 0x55;
  * UART1_SendData8(my_data);
  * @endcode
  */
void UART1_SendData8(u8 Data)
{
  /* Transmit Data */
  UART1->DR = Data;
}

/**
  * @brief Transmits 9 bit data through the UART1 peripheral.
  * @par Full description:
  * Transmits 9 bit data through the UART1 peripheral.
  * @param[in] Data: the data to transmit.
  * @retval void None
  * @par Required preconditions:
  * UART1_Cmd(ENABLE);
  * @par Called functions:
  * None
  * @par Example:
  * Send 0x103 using UART1
   * @code
  * u16 my_data = 0x103;
  * UART1_SendData9(my_data);
  * @endcode
  */
void UART1_SendData9(u16 Data)
{
  UART1->CR1 &= ((u8)~UART1_CR1_T8);                  /**< Clear the transmit data bit 8     */
  UART1->CR1 |= (u8)(((u8)(Data >> 2)) & UART1_CR1_T8); /**< Write the transmit data bit [8]   */
  UART1->DR   = (u8)(Data);                    /**< Write the transmit data bit [0:7] */

}
/**
  * @brief Transmits break characters.
  * @par Full description:
  * Transmits break characters on the UART1 peripheral.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * UART1_SendBreak();
  * @endcode
  */
void UART1_SendBreak(void)
{
  UART1->CR2 |= UART1_CR2_SBK;
}
/**
  * @brief Sets the address of the UART1 node.
  * @par Full description:
  * Sets the address of the UART1 node.
  * @param[in] UART1_Address: Indicates the address of the UART1 node.
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Set the address of the UART1 node at 0x04
  * @code
  * u8 my_UART1_Address = 0x04;
  * UART1_SetAddress(my_UART1_Address);
  * @endcode
  */

void UART1_SetAddress(u8 UART1_Address)
{
  /*assert_param for x UART1_Address*/
  assert_param(IS_UART1_Address_OK(UART1_Address));

  /* Clear the UART1 address */
  UART1->CR4 &= ((u8)~UART1_CR4_ADD);
  /* Set the UART1 address node */
  UART1->CR4 |= UART1_Address;
}
/**
  * @brief Checks whether the specified UART1 flag is set or not.
  * @par Full description:
  * Checks whether the specified UART1 flag is set or not.
  * @param[in] UART1_FLAG specifies the flag to check.
  * This parameter can be any of the @ref UART1_Flag_TypeDef enumeration.
  * @retval FlagStatus (SET or RESET)
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * Check the status of TC flag.
  * @code
  * FlagStatus TC_flag;
  * TC_Flag = UART1_GetFlagStatus(UART1_FLAG_TC);
  * @endcode
  */

FlagStatus UART1_GetFlagStatus(UART1_Flag_TypeDef UART1_FLAG)
{
  FlagStatus status = RESET;

  /* Check parameters */
  assert_param(IS_UART1_FLAG_OK(UART1_FLAG));

  /* Check the status of the specified UART1 flag*/
  if (UART1_FLAG == UART1_FLAG_LBD)
  {
    if ((UART1->CR4 & (u8)UART1_FLAG) != (u8)0x00)
    {
      /* UART1_FLAG is set*/
      status = SET;
    }
    else
    {
      /* UART1_FLAG is reset*/
      status = RESET;
    }
  }
  else if (UART1_FLAG == UART1_FLAG_SBK)
  {
    if ((UART1->CR2 & (u8)UART1_FLAG) != (u8)0x00)
    {
      /* UART1_FLAG is set*/
      status = SET;
    }
    else
    {
      /* UART1_FLAG is reset*/
      status = RESET;
    }
  }
  else if ((UART1_FLAG == UART1_FLAG_LHD) || (UART1_FLAG == UART1_FLAG_LSF))
  {
    if ((UART1->CR5 & (u8)UART1_FLAG) != (u8)0x00)
    {
      /* UART1_FLAG is set*/
      status = SET;
    }
    else
    {
      /* UART1_FLAG is reset*/
      status = RESET;
    }
  }
  else
  {
    if ((UART1->SR & (u8)UART1_FLAG) != (u8)0x00)
    {
      /* UART1_FLAG is set*/
      status = SET;
    }
    else
    {
      /* UART1_FLAG is reset*/
      status = RESET;
    }
  }

  /* Return the UART1_FLAG status*/
  return  status;
}
/**
 * @brief Clears the UART1 flags.
 * @par Full description:
 * Clears the UART1 flags.
 * @param[in] UART1_FLAG specifies the flag to clear
 * This parameter can be any combination of the following values:
 *   - UART1_FLAG_LBD: LIN Break detection flag.
 *   - UART1_FLAG_LHDF: LIN Header detection flag.
 *   - UART1_FLAG_LSF: LIN synchrone field flag.
 *   - UART1_FLAG_RXNE: Receive data register not empty flag.
 * @par Notes:
 *   - PE (Parity error), FE (Framing error), NE (Noise error), ORE (OverRun error)
 *     and IDLE (Idle line detected) flags are cleared by software sequence: a read
 *     operation to UART1_SR register (UART1_GetFlagStatus())followed by a read operation
 *     to UART1_DR register(UART1_ReceiveData8() or UART1_ReceiveData9()).
 *   - RXNE flag can be also cleared by a read to the UART1_DR register
 *     (UART1_ReceiveData8()or UART1_ReceiveData9()).
 *   - TC flag can be also cleared by software sequence: a read operation to UART1_SR
 *     register (UART1_GetFlagStatus()) followed by a write operation to UART1_DR register
 *     (UART1_SendData8() or UART1_SendData9()).
 *   - TXE flag is cleared only by a write to the UART1_DR register (UART1_SendData8() or
 *     UART1_SendData9()).
 *   - SBK flag is cleared during the stop bit of break.
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * None
 * @par Example:
 * Clear the UART1 LBD flag
 * @code
 * UART1_ClearFlag(UART1_FLAG_LBD);
 * @endcode
 */

void UART1_ClearFlag(UART1_Flag_TypeDef UART1_FLAG)
{
  assert_param(IS_UART1_CLEAR_FLAG_OK(UART1_FLAG));

  /*< Clear the Receive Register Not Empty flag */
  if (UART1_FLAG == UART1_FLAG_RXNE)
  {
    UART1->SR = (u8)~(UART1_SR_RXNE);
  }
  /*< Clear the LIN Break Detection flag */
  else if (UART1_FLAG == UART1_FLAG_LBD)
  {
    UART1->CR4 &= (u8)(~UART1_CR4_LBD);
  }
  /*< Clear the LIN Header Detection Flag */
  else if (UART1_FLAG == UART1_FLAG_LHD)
  {
    UART1->CR5 &= (u8)(~UART1_CR5_LHD);
  }
  /*< Clear the LIN Synch Field flag */
  else
  {
    UART1->CR5 &= (u8)(~UART1_CR5_LSF);
  }

}

/**
  * @brief Checks whether the specified UART1 interrupt has occurred or not.
  * @par Full description:
  * Checks whether the specified UART1 interrupt has occurred or not.
  * @param[in] UART1_IT: Specifies the UART1 interrupt pending bit to check.
  * This parameter can be one of the following values:
  *   - UART1_IT_LBD:  LIN Break detection interrupt
  *   - UART1_IT_TXE:  Tansmit Data Register empty interrupt
  *   - UART1_IT_TC:   Transmission complete interrupt
  *   - UART1_IT_RXNE: Receive Data register not empty interrupt
  *   - UART1_IT_IDLE: Idle line detection interrupt
  *   - UART1_IT_ORE:  OverRun Error interrupt
  *   - UART1_IT_PE:   Parity Error interrupt
  * @retval
  * ITStatus The new state of UART1_IT (SET or RESET).
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  * @par Example:
  * @code
  * IT_Status Status;
  * Status = UART1_GetITStatus(UART1_IT_TC);
  * @endcode
  */
ITStatus UART1_GetITStatus(UART1_IT_TypeDef UART1_IT)
{
  ITStatus pendingbitstatus = RESET;
  u8 itpos = 0;
  u8 itmask1 = 0;
  u8 itmask2 = 0;
  u8 enablestatus = 0;

  /* Check parameters */
  assert_param(IS_UART1_GET_IT_OK(UART1_IT));

  /* Get the UART1 IT index*/
  itpos = (u8)((u8)1 << (u8)((u8)UART1_IT & (u8)0x0F));
  /* Get the UART1 IT index*/
  itmask1 = (u8)((u8)UART1_IT >> (u8)4);
  /* Set the IT mask*/
  itmask2 = (u8)((u8)1 << itmask1);



  /* Check the status of the specified UART1 pending bit*/
  if (UART1_IT == UART1_IT_PE)
  {
    /* Get the UART1_ITPENDINGBIT enable bit status*/
    enablestatus = (u8)((u8)UART1->CR1 & itmask2);
    /* Check the status of the specified UART1 interrupt*/

    if (((UART1->SR & itpos) != (u8)0x00) && enablestatus)
    {
      /* Interrupt occurred*/
      pendingbitstatus = SET;
    }
    else
    {
      /* Interrupt not occurred*/
      pendingbitstatus = RESET;
    }
  }

  else if (UART1_IT == UART1_IT_LBD)
  {
    /* Get the UART1_IT enable bit status*/
    enablestatus = (u8)((u8)UART1->CR4 & itmask2);
    /* Check the status of the specified UART1 interrupt*/
    if (((UART1->CR4 & itpos) != (u8)0x00) && enablestatus)
    {
      /* Interrupt occurred*/
      pendingbitstatus = SET;
    }
    else
    {
      /* Interrupt not occurred*/
      pendingbitstatus = RESET;
    }
  }
  else if (UART1_IT == UART1_IT_LHD)
  {
    /* Get the UART1_IT enable bit status*/
    enablestatus = (u8)((u8)UART1->CR5 & itmask2);
    /* Check the status of the specified UART1 interrupt*/
    if (((UART1->CR5 & itpos) != (u8)0x00) && enablestatus)
    {
      /* Interrupt occurred*/
      pendingbitstatus = SET;
    }
    else
    {
      /* Interrupt not occurred*/
      pendingbitstatus = RESET;
    }
  }
  else
  {
    /* Get the UART1_IT enable bit status*/
    enablestatus = (u8)((u8)UART1->CR2 & itmask2);
    /* Check the status of the specified UART1 interrupt*/
    if (((UART1->SR & itpos) != (u8)0x00) && enablestatus)
    {
      /* Interrupt occurred*/
      pendingbitstatus = SET;
    }
    else
    {
      /* Interrupt not occurred*/
      pendingbitstatus = RESET;
    }
  }
  /* Return the UART1_IT status*/
  return  pendingbitstatus;
}

/**
 * @brief Clears the UART1 pending flags.
 * @par Full description:
 * Clears the UART1 pending bit.
 * @param[in] UART1_IT specifies the pending bit to clear
 * This parameter can be one of the following values:
 *   - UART1_IT_LBD:  LIN Break detection interrupt
 *   - UART1_IT_LHDF:  LIN Header detection interrupt
 *   - UART1_IT_RXNE: Receive Data register not empty interrupt.
 *
 * @par Notes:
 *   - PE (Parity error), FE (Framing error), NE (Noise error), ORE (OverRun error) and
 *     IDLE (Idle line detected) pending bits are cleared by software sequence: a read
 *     operation to UART1_SR register (UART1_GetITStatus()) followed by a read operation
 *     to UART1_DR register (UART1_ReceiveData8() or UART1_ReceiveData9() ).
 *   - RXNE pending bit can be also cleared by a read to the UART1_DR register
 *     (UART1_ReceiveData8() or UART1_ReceiveData9() ).
 *   - TC (Transmit complet) pending bit can be cleared by software sequence: a read
 *     operation to UART1_SR register (UART1_GetITStatus()) followed by a write operation
 *     to UART1_DR register (UART1_SendData8()or UART1_SendData9()).
 *   - TXE pending bit is cleared only by a write to the UART1_DR register
 *     (UART1_SendData8() or UART1_SendData9()).
 * @retval void None
 * @par Required preconditions:
 * None
 * @par Called functions:
 * None
 * @par Example:
 * Clear the UART1 LBD pending bit
 * @code
 * UART1_ClearITPendingBit(UART1_IT_LBD);
 * @endcode
 */
void UART1_ClearITPendingBit(UART1_IT_TypeDef UART1_IT)
{
  u8 dummy = 0;
  assert_param(IS_UART1_CLEAR_IT_OK(UART1_IT));

  /*< Clear the Receive Register Not Empty pending bit */
  if (UART1_IT == UART1_IT_RXNE)
  {
    UART1->SR = (u8)~(UART1_SR_RXNE);
  }
  /*< Clear the LIN Break Detection pending bit */
  else if (UART1_IT == UART1_IT_LBD)
  {
    UART1->CR4 &= (u8)~(UART1_CR4_LBD);
  }
  /*< Clear the LIN Header Detection pending bit */
  else
  {
    UART1->CR5 &= (u8)(~UART1_CR5_LHD);
  }
}
/**
  * @}
  */

/******************* (C) COPYRIGHT 2008 STMicroelectronics *****END OF FILE****/
